import datetime
import random
import re
import os

from hashlib import sha1

from django.conf import settings
from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from django.template.loader import render_to_string


class Host(models.Model):
    hostname = models.CharField(max_length=255, db_index=True, unique=True)
    mac = models.CharField(max_length=17, unique=True)
    created = models.DateTimeField(auto_now_add=datetime.datetime.now())
    disk_count = models.IntegerField(max_length=2, default=2)
    disk_regex = models.CharField(max_length=255, default='', null=True)
    ipmi_host = models.CharField(max_length=255, default='', null=True)
    ipmi_user = models.CharField(max_length=255, default='', null=True)
    ipmi_pass = models.CharField(max_length=255, default='', null=True)

    active_profile = models.CharField(max_length=255, default='', null=True)

    def get_dashed_mac(self):
        return '01-%s' % re.sub(':', '-', self.mac.lower())

    @property
    def status(self):
        installs = Installation.objects.filter(host=self)
        if installs.count() == 0:
            return 'unknown'
        return installs[0].status

    @property
    def color(self):
        st = {
            'pending_installation': 'active',
            'installing': 'info',
            'configuring': 'warning',
            'done': 'success',
            'error': 'danger',
            'unknown': '',
        }
        return st[self.status]


class Installation(models.Model):
    hash = models.CharField(max_length=40, unique=True, default='')
    profile = models.CharField(max_length=255)
    host = models.ForeignKey(Host, related_name='installations')
    created = models.DateTimeField(
        auto_now_add=True, default=datetime.datetime.now())
    install_conf = models.TextField(default='')
    pxeboot_conf = models.TextField(default='')
    first_boot = models.TextField(default='')
    status = models.CharField(
        choices=map(lambda x: (x, x), (
            'pending_installation',
            'installing',
            'configuring',
            'done',
            'error',
            'aborted',
        )),
        max_length='40', default='unknown')

    class Meta:
        ordering = ['-created']

    def __unicode__(self):
        return self.profile

    @classmethod
    def create(cls, host, profile):
        install = cls(host=host, profile=profile)
        install.hash = sha1('%s:%s:%s' % (
            host.get_dashed_mac(),
            profile, random.random())).hexdigest()
        install.pxeboot()
        install.preseed()
        install.firstboot()
        return install

    def preseed(self):
        host = self.host
        self.install_conf = render_to_string(
            'profiles/%s/preseed.cfg' % self.profile,
            {
                'hostname': host.hostname,
                'domain': '.'.join(host.hostname.split('.')[1:]),
                'repos': settings.ADDITIONAL_REPOS,
                'mirror_url': settings.MIRROR_URL,
                'firstboot_script_url': '%s/api/firstboot/%s.sh' % (
                    settings.PXETOOL_URL,
                    self.hash),
                'root_password_hash': settings.ROOT_PASSWORD_HASH,
                'disk_regex': host.disk_regex,
                'disk_count': host.disk_count,
            })
        return self.install_conf

    def pxeboot(self):
        host = self.host
        self.pxeboot_conf = render_to_string(
            'profiles/%s/pxelinux.cfg' % self.profile,
            {
                'hostname': host.hostname,
                'domain': host.hostname.partition('.')[2],
                'preseed_url': '%s/api/preseed/%s.cfg' % (
                    settings.PXETOOL_URL,
                    self.hash
                )
            })
        return self.pxeboot_conf

    def firstboot(self):
        self.first_boot = render_to_string(
            'profiles/%s/firstboot.sh' % self.profile,
            {
                'firstboot_checkpoint': '%s/api/firstboot/%s/checkpoint' % (
                    settings.PXETOOL_URL,
                    self.hash),
                'firstboot_done': '%s/api/firstboot/%s/done' % (
                    settings.PXETOOL_URL,
                    self.hash),
                'firstboot_error': '%s/api/firstboot/%s/error' % (
                    settings.PXETOOL_URL,
                    self.hash),
                'puppet_master': settings.PUPPET_MASTER,
            })
        return self.first_boot

    def delete_pxe(self):
        host = self.host
        pxelinux_cfg = os.path.join(
            settings.PXELINUX_ROOT,
            'pxelinux.cfg',
            host.get_dashed_mac())
        if os.path.exists(pxelinux_cfg):
            try:
                os.unlink(pxelinux_cfg)
            except OSError:
                return False
        return True

    def abort(self):
        if self.delete_pxe():
            self.status = 'aborted'
            self.save()

    def create_files(self):
        host = self.host
        pxelinux_cfg = os.path.join(
            settings.PXELINUX_ROOT,
            'pxelinux.cfg',
            host.get_dashed_mac())
        fp = open(pxelinux_cfg, 'w+')
        fp.write(self.pxeboot_conf)
        fp.close()


@receiver(pre_delete, sender=Installation)
def cleanup(sender, instance, **kwargs):
    instance.delete_pxe()
